/* Based on https://css-tricks.com/snippets/css/nested-expandable-folders/
   Reused with permission from https://css-tricks.com/license/
*/

.h5glance-css-treeview ul,
.h5glance-css-treeview li
{
	padding: 0;
	margin: 0;
    margin-left: 12px;
	list-style: none;
}

/* Override padding from Jupyter CSS */
.rendered_html .h5glance-css-treeview ul {
    padding: 0;
}

/* Make the checkbox itself invisible */
.h5glance-css-treeview input
{
	position: absolute;
	opacity: 0;
}

.h5glance-css-treeview
{
    font-family: sans-serif;
	-moz-user-select: none;
	-webkit-user-select: none;
	user-select: none;
}

.h5glance-css-treeview code {
    font-size: 12pt;
    font-family: "courier", monospace;
}

.h5glance-css-treeview a.h5glance-dataset-copylink
{
    text-decoration: none;
}

/* These next two bits drive the expanding/collapsing behaviour */
.h5glance-css-treeview input + label + ul
{
	display: none;
}

.h5glance-css-treeview input:checked:not(:disabled) + label + ul
{
	display: block;
}

.h5glance-css-treeview label,
.h5glance-css-treeview label::before
{
	display: inline-block;
	height: 16px;
	line-height: 16px;,
	vertical-align: middle;
}

.h5glance-css-treeview label
{
    margin-left: -1.5em;
}

.h5glance-css-treeview label::before
{
	content: "+";
	width: 1.5em;
	vertical-align: middle;
}

.h5glance-css-treeview input:checked + label::before
{
    content: "–";
}

/* webkit adjacent element selector bugfix */
@media screen and (-webkit-min-device-pixel-ratio:0)
{
	.h5glance-css-treeview
	{
		-webkit-animation: webkit-adjacent-element-selector-bugfix infinite 1s;
	}

	@-webkit-keyframes webkit-adjacent-element-selector-bugfix
	{
		from
		{
			padding: 0;
		}
		to
		{
			padding: 0;
		}
	}
}

.h5glance-css-treeview li.h5glance-dataset {
    margin-bottom: 3px;
}

.h5glance-dataset-name {
    font-weight: bold;
}
