#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['h5glance']

package_data = \
{'': ['*']}

install_requires = \
['h5py', 'htmlgen']

entry_points = \
{'console_scripts': ['h5glance = h5glance.terminal:main',
                     'h5glance-html = h5glance.html_cli:main']}

setup(name='h5glance',
      version='0.3',
      description='Explore HDF5 files in an HTML view',
      author='Thomas Kluyver',
      author_email='thomas@kluyver.me.uk',
      url='https://github.com/takluyver/h5glance',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.5',
     )
